@@ -25,8 +25,6 @@
 #include "archive.h"
 #include "operators.h"
 #include "utils.h"
-#include "clifford.h"
-#include "ncmul.h"
 #include "constant.h"
 #include "infinity.h"
 #include "compiler.h"
@@ -354,31 +352,17 @@ int add::ldegree(const ex & s) const
 	return deg;
 }
 
-ex add::coeff(const ex & s, int n) const
+ex add::coeff(const ex & s, const ex & n) const
 {
 	epvector coeffseq;
-	epvector coeffseq_cliff;
-	int rl = clifford_max_label(s);
-	bool do_clifford = (rl != -1);
-	bool nonscalar = false;
-
-	// Calculate sum of coefficients in each term
         for (const auto & elem : seq) {
 		ex restcoeff = elem.rest.coeff(s, n);
  		if (!restcoeff.is_zero()) {
- 			if (do_clifford) {
- 				if (clifford_max_label(restcoeff) == -1) {
-                                        coeffseq_cliff.push_back(expair(ncmul(restcoeff, dirac_ONE(rl)), elem.coeff));
-				} else {
-                                        coeffseq_cliff.push_back(expair(restcoeff, elem.coeff));
-					nonscalar = true;
- 				}
-			}
 			coeffseq.push_back(expair(restcoeff, elem.coeff));
 		}
 	}
 
-	return (new add(nonscalar ? coeffseq_cliff : coeffseq,
+	return (new add(coeffseq,
 	                n==0 ? overall_coeff : *_num0_p))->setflag(status_flags::dynallocated);
 }
 
@@ -391,7 +375,7 @@ ex add::coeff(const ex & s, int n) const
  *  @param level cut-off in recursive evaluation */
 ex add::eval(int level) const
 {
-        if ((level == 1) && (flags & status_flags::evaluated)) {
+        if ((level == 1) and is_evaluated()) {
                 GINAC_ASSERT(seq.size()>0);
                 GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_zero());
                 return *this;
@@ -534,14 +518,6 @@ ex add::imag_part() const
 		-> setflag(status_flags::dynallocated);
 }
 
-ex add::eval_ncmul(const exvector & v) const
-{
-	if (seq.empty())
-		return inherited::eval_ncmul(v);
-	else
-		return seq.begin()->rest.eval_ncmul(v);
-}    
-
 // protected
 
 /** Implementation of ex::diff() for a sum. It differentiates each term.
