@@ -27,7 +27,6 @@
 #include "add.h"
 #include "symbol.h"
 #include "lst.h"
-#include "ncmul.h"
 #include "relational.h"
 #include "operators.h"
 #include "wildcard.h"
@@ -137,11 +136,11 @@ void basic::print(const print_context & c, unsigned level) const
 void basic::print_dispatch(const registered_class_info & ri, const print_context & c, unsigned level) const
 {
 	// Double dispatch on object type and print_context type
-	const registered_class_info * reg_info = &ri;
+	const registered_class_info * r_info = &ri;
 	const print_context_class_info * pc_info = &c.get_class_info();
 
 next_class:
-	const std::vector<print_functor> & pdt = reg_info->options.get_print_dispatch_table();
+	const std::vector<print_functor> & pdt = r_info->options.get_print_dispatch_table();
 
 next_context:
 	unsigned id = pc_info->options.get_id();
@@ -155,9 +154,9 @@ next_context:
 		}
 
 		// Method still not found, try parent class
-		const registered_class_info * parent_reg_info = reg_info->get_parent();
+		const registered_class_info * parent_reg_info = r_info->get_parent();
 		if (parent_reg_info != nullptr) {
-			reg_info = parent_reg_info;
+			r_info = parent_reg_info;
 			pc_info = &c.get_class_info();
 			goto next_class;
 		}
@@ -346,12 +345,12 @@ int basic::ldegree(const ex & s) const
 }
 
 /** Return coefficient of degree n in object s. */
-ex basic::coeff(const ex & s, int n) const
+ex basic::coeff(const ex & s, const ex & n) const
 {
 	if (is_equal(ex_to<basic>(s)))
-		return n==1 ? _ex1 : _ex0;
+		return n.is_integer_one() ? _ex1 : _ex0;
 	else
-		return n==0 ? *this : _ex0;
+		return n.is_zero() ? *this : _ex0;
 }
 
 /** Sort expanded expression in terms of powers of some object(s).
@@ -459,20 +458,7 @@ ex basic::evalm() const
 	return *this;
 }
 
-/** Function object to be applied by basic::eval_integ(). */
-struct eval_integ_map_function : public map_function {
-	ex operator()(const ex & e) override { return eval_integ(e); }
-} map_eval_integ;
-
-/** Evaluate integrals, if result is known. */
-ex basic::eval_integ() const
-{
-	if (nops() == 0)
-		return *this;
-	else
-		return map(map_eval_integ);
-}
-
+#if 0
 /** Perform automatic symbolic evaluations on indexed expression that
  *  contains this object as the base expression. */
 ex basic::eval_indexed(const basic & i) const
@@ -524,7 +510,9 @@ bool basic::contract_with(exvector::iterator /*unused*/, exvector::iterator /*un
 {
 	// Do nothing
 	return false;
+
 }
+#endif
 
 /** Check whether the expression matches a given pattern. For every wildcard
  *  object in the pattern, an expression of the form "wildcard == matching_expression"
@@ -659,7 +647,7 @@ ex basic::diff(const symbol & s, unsigned nth) const
 		return ex(*this);
 	
 	// evaluate unevaluated *this before differentiating
-	if ((flags & status_flags::evaluated) == 0u)
+	if (not is_evaluated())
 		return ex(*this).diff(s, nth);
 	
 	ex ndiff = this->derivative(s);
@@ -671,12 +659,6 @@ ex basic::diff(const symbol & s, unsigned nth) const
 	return ndiff;
 }
 
-/** Return a vector containing the free indices of an expression. */
-exvector basic::get_free_indices() const
-{
-	return exvector(); // return an empty exvector
-}
-
 ex basic::conjugate() const
 {
 	return *this;
@@ -692,11 +674,6 @@ ex basic::imag_part() const
 	return imag_part_function(*this).hold();
 }
 
-ex basic::eval_ncmul(const exvector & v) const
-{
-	return hold_ncmul(v);
-}
-
 // protected
 
 /** Function object to be applied by basic::derivative(). */
@@ -782,7 +759,7 @@ long basic::calchash() const
 	}
 
 	// store calculated hash value only if object is already evaluated
-	if ((flags & status_flags::evaluated) != 0u) {
+	if (is_evaluated()) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
 	}
