@@ -25,7 +25,6 @@
 #include "symbol.h"
 #include "add.h"
 #include "mul.h"
-#include "ncmul.h"
 #include "numeric.h"
 #include "power.h"
 #include "lst.h"
@@ -584,7 +583,7 @@ void ex::coefficients(const ex & s, expairvec & vec) const
 
         if (is_exactly_a<add>(sub)) {
                 const add& addref = ex_to<add>(sub);
-                const ex& oc = addref.op(addref.nops()+1);
+                const numeric& oc = addref.get_overall_coeff();
                 if (not oc.is_zero())
                         vec.push_back(std::make_pair(oc, _ex0));
                 for (const auto& term : addref.seq) {
@@ -694,7 +693,7 @@ void ex::share(const ex & other) const
  *  @see ex::ex(const basic &) */
 ptr<basic> ex::construct_from_basic(const basic & other)
 {
-	if ((other.flags & status_flags::evaluated) == 0u) {
+	if (not other.is_evaluated()) {
 
 		// The object is not yet evaluated, so call eval() to evaluate
 		// the top level. This will return either
