@@ -33,6 +33,7 @@
 #include "symbol.h"
 #include "symmetry.h"
 #include "utils.h"
+#include "wildcard.h"
 
 #include <vector>
 #include <stdexcept>
@@ -113,6 +114,8 @@ static ex real_part_evalf(const ex & arg, PyObject* parent)
 
 static ex real_part_eval(const ex & arg)
 {
+        if (arg.info(info_flags::real))
+                return arg;
         ex pat = pow(wild(1), wild(2));
         lst l;
         if (arg.find(pat, l))
@@ -177,6 +180,8 @@ static ex imag_part_evalf(const ex & arg, PyObject* parent)
 
 static ex imag_part_eval(const ex & arg)
 {
+        if (arg.info(info_flags::real))
+                return _ex0;
         ex pat = pow(wild(1), wild(2));
         lst l;
         if (arg.find(pat, l))
@@ -390,9 +395,8 @@ static ex unit_step_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return unit_step_evalf(arg, nullptr);
 	
-	else if (is_exactly_a<mul>(arg) &&
-	         is_exactly_a<numeric>(arg.op(arg.nops()-1))) {
-		numeric oc = ex_to<numeric>(arg.op(arg.nops()-1));
+	else if (is_exactly_a<mul>(arg)) {
+		const numeric& oc = ex_to<mul>(arg).get_overall_coeff();
 		if (oc.is_real()) {
 			if (oc > 0)
 				// step(42*x) -> step(x)
@@ -475,9 +479,8 @@ static ex heaviside_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return heaviside_evalf(arg, nullptr);
 	
-	else if (is_exactly_a<mul>(arg) &&
-	         is_exactly_a<numeric>(arg.op(arg.nops()-1))) {
-		numeric oc = ex_to<numeric>(arg.op(arg.nops()-1));
+	else if (is_exactly_a<mul>(arg)) {
+		const numeric& oc = ex_to<mul>(arg).get_overall_coeff();
 		if (oc.is_real()) {
 			if (oc > 0)
 				// step(42*x) -> step(x)
@@ -554,9 +557,8 @@ static ex csgn_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return csgn(ex_to<numeric>(arg));
 	
-	else if (is_exactly_a<mul>(arg) &&
-	         is_exactly_a<numeric>(arg.op(arg.nops()-1))) {
-		numeric oc = ex_to<numeric>(arg.op(arg.nops()-1));
+	else if (is_exactly_a<mul>(arg)) {
+		const numeric& oc = ex_to<mul>(arg).get_overall_coeff();
 		if (oc.is_real()) {
 			if (oc > 0)
 				// csgn(42*x) -> csgn(x)
@@ -742,8 +744,7 @@ static ex Order_eval(const ex & x)
 	} else if (is_exactly_a<mul>(x)) {
 		const mul &m = ex_to<mul>(x);
 		// O(c*expr) -> O(expr)
-		if (is_exactly_a<numeric>(m.op(m.nops() - 1)))
-			return Order(x / m.op(m.nops() - 1)).hold();
+		return Order(x / m.get_overall_coeff()).hold();
 	}
 	return Order(x).hold();
 }
@@ -788,4 +789,78 @@ REGISTER_FUNCTION(Order, eval_func(Order_eval).
                          real_part_func(Order_real_part).
                          imag_part_func(Order_imag_part));
 
+static ex cases_eval(const ex & arg)
+{
+        if (not is_exactly_a<exprseq>(arg))
+                throw std::runtime_error("cases argument not a sequence");
+        bool default_seen = false;
+        ex deflt;
+        for (auto case_ : arg) {
+                if (not is_exactly_a<exprseq>(case_)) {
+                        if (not default_seen) {
+                                deflt = case_;
+                                default_seen = true;
+                        }
+                        continue;
+                }
+                const exprseq& seq = ex_to<exprseq>(case_);
+                if (seq.nops() == 1) {
+                        if (not default_seen) {
+                                deflt = seq[0];
+                                default_seen = true;
+                        }
+                        continue;
+                }
+                if (seq.nops() != 2)
+                        throw std::runtime_error("cases case not a pair");
+                const ex& cond = seq[0];
+                if (is_exactly_a<numeric>(cond)) {
+                        if (not cond.is_zero()
+                            and not default_seen) {
+                                deflt = seq[1];
+                                default_seen = true;
+                        }
+                        continue;
+                }
+                if (is_exactly_a<relational>(cond)) {
+                        auto res = ex_to<relational>(cond).decide();
+                        if (res == relational::result::True)
+                                return seq[1];
+                        else if (res == relational::result::False)
+                                continue;
+                        // undecidable relation
+                        return cases(arg).hold();
+                }
+                throw std::runtime_error("cases with meaningless condition");
+        }
+        if (default_seen)
+                return deflt;
+        return cases(arg).hold();
+}
+
+static ex cases_evalf(const ex & arg, PyObject* parent)
+{
+        return cases(arg).hold();
+}
+
+static ex cases_conjugate(const ex & arg)
+{
+        return cases(arg).hold();
+}
+
+static ex cases_real_part(const ex & arg)
+{
+        return cases(arg).hold();
+}
+
+static ex cases_imag_part(const ex & arg)
+{
+        return cases(arg).hold();
+}
+
+REGISTER_FUNCTION(cases, eval_func(cases_eval).
+                       evalf_func(cases_evalf).
+                       conjugate_func(cases_conjugate).
+                       real_part_func(cases_real_part).
+                       imag_part_func(cases_imag_part));
 } // namespace GiNaC
