@@ -48,18 +48,35 @@ static bool has_pi(const ex & the_ex) {
                         return true;
         return false;
 }
+
+// helper function: returns whether the expression is a multiple of I
+static bool is_multiple_of_I(const ex & the_ex)
+{
+
+	if (is_exactly_a<numeric>(the_ex)
+	    and the_ex.real_part().is_zero())
+		return true;
+
+	if (is_exactly_a<mul>(the_ex)) {
+		for (size_t i=0; i < the_ex.nops(); ++i)
+			if (is_multiple_of_I(the_ex.op(i)))
+				return true;
+	}
+
+	if (is_exactly_a<add>(the_ex)) {
+		for (size_t i=0; i < the_ex.nops(); ++i)
+			if (!is_multiple_of_I(the_ex.op(i)))
+				return false;
+		return true;
+	}
+	return false;
+};
+
+
 //////////
 // sine (trigonometric function)
 //////////
 
-static ex sin_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return sin(ex_to<numeric>(x));
-	
-	return sin(x).hold();
-}
-
 static ex sin_eval(const ex & x)
 {
 	// sin(oo) -> error
@@ -79,7 +96,7 @@ static ex sin_eval(const ex & x)
         ex x_red;
         if (has_pi(x)) {
 
-                ex coef_pi = x.coeff(Pi).expand();
+                ex coef_pi = x.coeff(Pi,_ex1).expand();
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
@@ -195,6 +212,10 @@ static ex sin_eval(const ex & x)
         else
                 x_red = x;
 
+	// simplify sin(I*x) --> I*sinh(x)
+	if (is_multiple_of_I(x_red.expand()))
+		return I*sinh(x_red/I);
+
 	if (is_exactly_a<function>(x_red)) {
 		const ex &t = x_red.op(0);
 
@@ -261,7 +282,6 @@ static ex sin_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(sin, eval_func(sin_eval).
-                       evalf_func(sin_evalf).
                        derivative_func(sin_deriv).
                        real_part_func(sin_real_part).
                        imag_part_func(sin_imag_part).
@@ -272,14 +292,6 @@ REGISTER_FUNCTION(sin, eval_func(sin_eval).
 // cosine (trigonometric function)
 //////////
 
-static ex cos_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return cos(ex_to<numeric>(x));
-	
-	return cos(x).hold();
-}
-
 static ex cos_eval(const ex & x)
 {
 	// cos(oo) -> error
@@ -299,7 +311,7 @@ static ex cos_eval(const ex & x)
         ex x_red;
         if (has_pi(x)) {
 
-                ex coef_pi = x.coeff(Pi).expand();
+                ex coef_pi = x.coeff(Pi,_ex1).expand();
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
@@ -409,6 +421,10 @@ static ex cos_eval(const ex & x)
         else
                 x_red = x;
 
+	// simplify cos(I*x) --> cosh(x)
+	if (is_multiple_of_I(x_red.expand()))
+		return cosh(x_red/I);
+
 	if (is_exactly_a<function>(x_red)) {
 		const ex &t = x_red.op(0);
 
@@ -476,7 +492,6 @@ static ex cos_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(cos, eval_func(cos_eval).
-                       evalf_func(cos_evalf).
                        derivative_func(cos_deriv).
                        real_part_func(cos_real_part).
                        imag_part_func(cos_imag_part).
@@ -487,14 +502,6 @@ REGISTER_FUNCTION(cos, eval_func(cos_eval).
 // tangent (trigonometric function)
 //////////
 
-static ex tan_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return tan(ex_to<numeric>(x));
-	
-	return tan(x).hold();
-}
-
 static ex tan_eval(const ex & x)
 {
 	// tan(oo) -> error
@@ -514,7 +521,7 @@ static ex tan_eval(const ex & x)
         ex x_red;
         if (has_pi(x)) {
 
-                ex coef_pi = x.coeff(Pi).expand();
+                ex coef_pi = x.coeff(Pi,_ex1).expand();
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
@@ -628,6 +635,9 @@ static ex tan_eval(const ex & x)
         else
                 x_red = x;
 
+	if (is_multiple_of_I(x_red.expand()))
+		return I*tanh(x_red/I);
+
 	if (is_exactly_a<function>(x_red)) {
 		const ex &t = x_red.op(0);
 
@@ -716,7 +726,6 @@ static ex tan_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(tan, eval_func(tan_eval).
-                       evalf_func(tan_evalf).
                        derivative_func(tan_deriv).
                        series_func(tan_series).
                        real_part_func(tan_real_part).
@@ -728,19 +737,16 @@ REGISTER_FUNCTION(tan, eval_func(tan_eval).
 // cotangent (trigonometric function)
 //////////
 
-static ex cot_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x)) {
-                if (ex_to<numeric>(x).is_zero())
-                        return UnsignedInfinity;
-		return tan(ex_to<numeric>(x)).inverse();
-        }
-
-	return cot(x).hold();
-}
-
 static ex cot_eval(const ex & x)
 {
+	// This should be before the tests below, since multiplying infinity
+	// with other values raises runtime_errors
+	if (x.is_zero())
+		return UnsignedInfinity;
+
+	if (is_multiple_of_I(x.expand()))
+		return -I*coth(x/I);
+
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
                    
@@ -793,7 +799,7 @@ static ex cot_eval(const ex & x)
         // Reflection at Pi/2
         const ex ExOverPi = x/Pi;
         if(is_exactly_a<numeric>(ExOverPi)) {
-		ex coef_pi = x.coeff(Pi).expand();
+		ex coef_pi = x.coeff(Pi,_ex1).expand();
 		if (is_exactly_a<numeric>(coef_pi)) {
 			const numeric c = ex_to<numeric>(coef_pi);
 		        if (c.is_rational()) {
@@ -860,7 +866,6 @@ static ex cot_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(cot, eval_func(cot_eval).
-                       evalf_func(cot_evalf).
                        derivative_func(cot_deriv).
                        series_func(cot_series).
                        real_part_func(cot_real_part).
@@ -872,16 +877,11 @@ REGISTER_FUNCTION(cot, eval_func(cot_eval).
 // secant (trigonometric function)
 //////////
 
-static ex sec_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return cos(ex_to<numeric>(x)).inverse();
-
-	return sec(x).hold();
-}
-
 static ex sec_eval(const ex & x)
 {
+	if (is_multiple_of_I(x.expand()))
+		return sech(x/I);
+
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
@@ -981,7 +981,6 @@ static ex sec_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(sec, eval_func(sec_eval).
-                       evalf_func(sec_evalf).
                        derivative_func(sec_deriv).
                        series_func(sec_series).
                        real_part_func(sec_real_part).
@@ -993,20 +992,12 @@ REGISTER_FUNCTION(sec, eval_func(sec_eval).
 // cosecant (trigonometric function)
 //////////
 
-static ex csc_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x)) {
-                if (ex_to<numeric>(x).is_zero())
-                        return UnsignedInfinity;
-		return sin(ex_to<numeric>(x)).inverse();
-        }
-
-	return csc(x).hold();
-}
-
 static ex csc_eval(const ex & x)
 {
 
+	if (is_multiple_of_I(x.expand()))
+		return -I*csch(x/I);
+
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
@@ -1104,7 +1095,6 @@ static ex csc_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(csc, eval_func(csc_eval).
-                       evalf_func(csc_evalf).
                        derivative_func(csc_deriv).
                        series_func(csc_series).
                        real_part_func(csc_real_part).
@@ -1116,14 +1106,6 @@ REGISTER_FUNCTION(csc, eval_func(csc_eval).
 // inverse sine (arc sine)
 //////////
 
-static ex asin_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return asin(ex_to<numeric>(x));
-	
-	return asin(x).hold();
-}
-
 static ex asin_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -1194,7 +1176,6 @@ static ex asin_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(asin, eval_func(asin_eval).
-                        evalf_func(asin_evalf).
                         derivative_func(asin_deriv).
                         conjugate_func(asin_conjugate).
 			set_name("arcsin", "\\arcsin"));
@@ -1203,14 +1184,6 @@ REGISTER_FUNCTION(asin, eval_func(asin_eval).
 // inverse cosine (arc cosine)
 //////////
 
-static ex acos_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return acos(ex_to<numeric>(x));
-	
-	return acos(x).hold();
-}
-
 static ex acos_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -1281,7 +1254,6 @@ static ex acos_conjugate(const ex & x)
 
  
 REGISTER_FUNCTION(acos, eval_func(acos_eval).
-                        evalf_func(acos_evalf).
                         derivative_func(acos_deriv).
                         conjugate_func(acos_conjugate).
 			set_name("arccos", "\\arccos"));
@@ -1290,14 +1262,6 @@ REGISTER_FUNCTION(acos, eval_func(acos_eval).
 // inverse tangent (arc tangent)
 //////////
 
-static ex atan_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return atan(ex_to<numeric>(x));
-	
-	return atan(x).hold();
-}
-
 static ex atan_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -1410,7 +1374,6 @@ static ex atan_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(atan, eval_func(atan_eval).
-                        evalf_func(atan_evalf).
                         derivative_func(atan_deriv).
                         series_func(atan_series).
                         conjugate_func(atan_conjugate).
@@ -1537,6 +1500,7 @@ REGISTER_FUNCTION(atan2, eval_func(atan2_eval).
 // inverse cotangent (arc cotangent)
 //////////
 
+// Needed because there is no Python RR equivalent
 static ex acot_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
@@ -1605,6 +1569,7 @@ REGISTER_FUNCTION(acot, eval_func(acot_eval).
 // inverse secant (arc secant)
 //////////
 
+// Needed because there is no Python RR equivalent
 static ex asec_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
@@ -1669,6 +1634,7 @@ REGISTER_FUNCTION(asec, eval_func(asec_eval).
 // inverse cosecant (arc cosecant)
 //////////
 
+// Needed because there is no Python RR equivalent
 static ex acsc_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
