@@ -110,7 +110,12 @@ ex multiply_lcm(const ex &e, const numeric &lcm)
 		if (is_exactly_a<symbol>(e.op(0)))
 			return e * lcm;
 		else {
-			numeric root_of_lcm = lcm.power(ex_to<numeric>(e.op(1)).inverse());
+                        if (not is_exactly_a<numeric>(e.op(1)))
+				return e * lcm;
+			ex t = lcm.power(ex_to<numeric>(e.op(1)).inverse());
+                        if (not is_exactly_a<numeric>(t))
+				return e * lcm;
+			numeric root_of_lcm = ex_to<numeric>(t);
 			if (root_of_lcm.is_rational())
 				return pow(multiply_lcm(e.op(0), root_of_lcm), e.op(1));
 			else
