@@ -26,13 +26,11 @@
 #include "expairseq.h"
 #include "add.h"
 #include "mul.h"
-#include "ncmul.h"
 #include "numeric.h"
 #include "constant.h"
 #include "infinity.h"
 #include "operators.h"
 #include "inifcns.h" // for log() in power::derivative() and exp for printing
-#include "indexed.h"
 #include "symbol.h"
 #include "lst.h"
 #include "archive.h"
@@ -337,21 +335,6 @@ bool power::info(unsigned inf) const
                                  (exponent.info(info_flags::integer))) or
                                 (basis.info(info_flags::positive) and
                                  exponent.info(inf)));
-		case info_flags::has_indices: {
-			if ((flags & status_flags::has_indices) != 0u)
-				return true;
-			else if ((flags & status_flags::has_no_indices) != 0u)
-				return false;
-			else if (basis.info(info_flags::has_indices)) {
-				setflag(status_flags::has_indices);
-				clearflag(status_flags::has_no_indices);
-				return true;
-			} else {
-				clearflag(status_flags::has_indices);
-				setflag(status_flags::has_no_indices);
-				return false;
-			}
-		}
 	}
 	return inherited::info(inf);
 }
@@ -431,32 +414,24 @@ int power::ldegree(const ex & s) const
 		return 0;
 }
 
-ex power::coeff(const ex & s, int n) const
+ex power::coeff(const ex & s, const ex & n) const
 {
 	if (is_equal(ex_to<basic>(s)))
-		return n==1 ? _ex1 : _ex0;
+		return n.is_integer_one() ? _ex1 : _ex0;
 	else if (!basis.is_equal(s)) {
 		// basis not equal to s
-		if (n == 0)
+		if (n.is_zero())
 			return *this;
 		else
 			return _ex0;
 	} else {
 		// basis equal to s
-		if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_integer()) {
-			// integer exponent
-			int int_exp = ex_to<numeric>(exponent).to_int();
-			if (n == int_exp)
-				return _ex1;
-			else
-				return _ex0;
-		} else {
-			// non-integer exponents are treated as zero
-			if (n == 0)
-				return *this;
-			else
-				return _ex0;
-		}
+                if (n.is_zero())
+                        return _ex0;
+                else if (exponent.is_equal(n))
+                        return _ex1;
+                else
+                        return _ex0;
 	}
 }
 
@@ -477,7 +452,7 @@ ex power::coeff(const ex & s, int n) const
  *  @param level cut-off in recursive evaluation */
 ex power::eval(int level) const
 {
-	if ((level==1) && ((flags & status_flags::evaluated) != 0u))
+	if (level == 1 and is_evaluated())
 		return *this;
 	else if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
@@ -751,7 +726,33 @@ ex power::eval(int level) const
 					}
 				}
 			}
-		}
+                        else {
+                                // (x*y^(m/n)*z)^(r/s) ---> y^t*(x*z)^(r/s), if t integer
+                                exvector outer, inner;
+                                for (size_t i=0; i<mulref.nops(); ++i) {
+                                        const ex& fac = mulref.op(i);
+                                        ex pfac = power(fac, eexponent);
+                                        if (not is_exactly_a<power>(pfac)
+                                            or ex_to<power>(pfac).exponent.info(info_flags::integer))
+                                                outer.push_back(pfac);
+                                        else
+                                                inner.push_back(fac);
+                                }
+                                if (outer.size() > 0) {
+                                        ex outex, innex;
+                                        if (outer.size() == 1)
+                                                outex = outer[0];
+                                        else
+                                                outex = mul(outer).hold();
+                                        if (inner.size() == 1)
+                                                innex = inner[0];
+                                        else
+                                                innex = mul(inner).hold();
+                                        ex p = power(innex, eexponent).hold();
+                                        return (new mul(outex, p))->setflag(status_flags::dynallocated | status_flags::evaluated);
+                                }
+                        }
+                }
 	}
 
 	// Reduce x^(c/log(x)) to exp(c) if x is positive
@@ -847,11 +848,6 @@ ex power::subs(const exmap & m, unsigned options) const
 	return subs_one_level(m, options);
 }
 
-ex power::eval_ncmul(const exvector & v) const
-{
-	return inherited::eval_ncmul(v);
-}
-
 ex power::conjugate() const
 {
 	// conjugate(pow(x,y))==pow(conjugate(x),conjugate(y)) unless on the
@@ -1215,7 +1211,11 @@ private:
 		explicit coolmulti(const std::vector<int>& partition)
 		  : head(nullptr), i(nullptr), after_i(nullptr)
 		{
-			for (unsigned n = 0; n < partition.size(); ++n) {
+                        if (partition.size() <= 0)
+                                throw std::runtime_error("can't happn in coolmuli");
+                        head = new element(partition[0], head);
+                        i = head;
+			for (unsigned n = 1; n < partition.size(); ++n) {
 				head = new element(partition[n], head);
 				if (n <= 1)
 					i = head;
@@ -1524,22 +1524,6 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 		return _ex1;
 	}
 
-	// do not bother to rename indices if there are no any.
-	if (((options & expand_options::expand_rename_idx) == 0u) 
-			&& m.info(info_flags::has_indices))
-		options |= expand_options::expand_rename_idx;
-	// Leave it to multiplication since dummy indices have to be renamed
-	if (((options & expand_options::expand_rename_idx) != 0u) &&
-		(get_all_dummy_indices(m).size() > 0) && n.is_positive()) {
-		ex result = m;
-		exvector va = get_all_dummy_indices(m);
-		sort(va.begin(), va.end(), ex_is_less());
-
-		for (int i=1; i < n.to_int(); i++)
-			result *= rename_dummy_indices_uniquely(va, m);
-		return result;
-	}
-
 	epvector distrseq;
 	distrseq.reserve(m.seq.size());
 	bool need_reexpand = false;
